@@ -132,6 +132,10 @@ typedef struct _CompWalker        CompWalker;
 #define CompWindowTypeFullscreenMask   (1 << 15)
 #define CompWindowTypeUnknownMask      (1 << 16)
 
+#define NO_FOCUS_MASK (CompWindowTypeDesktopMask | \
+		       CompWindowTypeDockMask    | \
+		       CompWindowTypeSplashMask)
+
 #define CompWindowStateModalMask	    (1 <<  0)
 #define CompWindowStateStickyMask	    (1 <<  1)
 #define CompWindowStateMaximizedVertMask    (1 <<  2)
@@ -2310,7 +2314,8 @@ typedef void (*FiniPluginForWindowProc) (CompPlugin *plugin,
 typedef enum {
     CompStackingUpdateModeNone = 0,
     CompStackingUpdateModeNormal,
-    CompStackingUpdateModeAboveFullscreen
+    CompStackingUpdateModeAboveFullscreen,
+    CompStackingUpdateModeInitialMap
 } CompStackingUpdateMode;
 
 struct _CompWindowExtents {
@@ -2783,7 +2788,9 @@ setWindowUserTime (CompWindow *w,
 		   Time       time);
 
 Bool
-allowWindowFocus (CompWindow *w);
+allowWindowFocus (CompWindow   *w,
+		  unsigned int noFocusMask,
+		  Time         timestamp);
 
 void
 unredirectWindow (CompWindow *w);
