@@ -25,7 +25,7 @@
 
 #include <compiz.h>
 
-#define CUBE_ABIVERSION 20070621
+#define CUBE_ABIVERSION 20070827
 
 #define CUBE_MOMODE_AUTO  0
 #define CUBE_MOMODE_MULTI 1
@@ -33,11 +33,11 @@
 #define CUBE_MOMODE_LAST  CUBE_MOMODE_ONE
 
 #define CUBE_DISPLAY_OPTION_ABI    0
-#define CUBE_DISPLAY_OPTION_INDEX      1
-#define CUBE_DISPLAY_OPTION_UNFOLD_KEY 2
-#define CUBE_DISPLAY_OPTION_NEXT_KEY   3
-#define CUBE_DISPLAY_OPTION_PREV_KEY   4
-#define CUBE_DISPLAY_OPTION_NUM        5
+#define CUBE_DISPLAY_OPTION_INDEX  1
+#define CUBE_DISPLAY_OPTION_UNFOLD 2
+#define CUBE_DISPLAY_OPTION_NEXT   3
+#define CUBE_DISPLAY_OPTION_PREV   4
+#define CUBE_DISPLAY_OPTION_NUM    5
 
 typedef struct _CubeDisplay {
     int	screenPrivateIndex;
@@ -167,6 +167,8 @@ typedef struct _CubeScreen {
 
     Bool cleared[64];
 
+    Bool capsPainted[64];
+
     Bool fullscreenOutput;
 
     float outputXScale;
