@@ -35,8 +35,8 @@ static CompMetadata cloneMetadata;
 
 static int displayPrivateIndex;
 
-#define CLONE_DISPLAY_OPTION_INITIATE_BUTTON 0
-#define CLONE_DISPLAY_OPTION_NUM             1
+#define CLONE_DISPLAY_OPTION_INITIATE 0
+#define CLONE_DISPLAY_OPTION_NUM      1
 
 typedef struct _CloneDisplay {
     int		    screenPrivateIndex;
@@ -699,7 +699,7 @@ cloneGetDisplayOptions (CompPlugin  *plugin,
 static Bool
 cloneSetDisplayOption (CompPlugin      *plugin,
 		       CompDisplay     *display,
-		       const char      *name,
+		       char	       *name,
 		       CompOptionValue *value)
 {
     CompOption *o;
@@ -714,7 +714,8 @@ cloneSetDisplayOption (CompPlugin      *plugin,
 }
 
 static const CompMetadataOptionInfo cloneDisplayOptionInfo[] = {
-    { "initiate_button", "button", 0, cloneInitiate, cloneTerminate }
+    { "initiate", "action", "<allowed button=\"true\"/>", cloneInitiate,
+      cloneTerminate }
 };
 
 static Bool
