@@ -679,7 +679,7 @@ cubeLoadBackground (CompScreen *s,
 static Bool
 cubeSetScreenOption (CompPlugin      *plugin,
 		     CompScreen      *screen,
-		     const char	     *name,
+		     char	     *name,
 		     CompOptionValue *value)
 {
     CompOption *o;
@@ -861,6 +861,7 @@ cubePreparePaintScreen (CompScreen *s,
     }
 
     memset (cs->cleared, 0, sizeof (Bool) * s->nOutputDev);
+    memset (cs->capsPainted, 0, sizeof (Bool) * s->nOutputDev);
 
     /* Transparency handling */
     if (cs->rotationState == RotationManual ||
@@ -1362,12 +1363,12 @@ cubePaintTransformedOutput (CompScreen		    *s,
 			    unsigned int	    mask)
 {
     ScreenPaintAttrib sa = *sAttrib;
-    float	      xRotate, vRotate;
+    float	      xRotate, vRotate, rRotate;
     int		      hsize, xMove = 0;
     float	      size;
     GLenum            filter = s->display->textureFilter;
     PaintOrder        paintOrder;
-    Bool	      clear;
+    Bool	      capsPainted;
     Bool              wasCulled = FALSE;
     Bool              topDir, bottomDir;
     int output = 0;
@@ -1387,12 +1388,7 @@ cubePaintTransformedOutput (CompScreen		    *s,
     hsize = s->hsize * cs->nOutput;
     size  = hsize;
 
-    if (cs->desktopOpacity != OPAQUE)
-    {
-	wasCulled = glIsEnabled (GL_CULL_FACE);
-	if (wasCulled)
-	    glDisable (GL_CULL_FACE);
-    }
+    wasCulled = glIsEnabled (GL_CULL_FACE);
 
     if (!cs->fullscreenOutput)
     {
@@ -1424,10 +1420,10 @@ cubePaintTransformedOutput (CompScreen		    *s,
     sa.xRotate += xRotate;
     sa.vRotate += vRotate;
 
-    clear = cs->cleared[output];
-    if (!clear)
+    if (!cs->cleared[output])
     {
-	(*cs->clearTargetOutput) (s, xRotate, vRotate);
+	rRotate = xRotate - ((360.0f / hsize) * (s->x * cs->nOutput));
+	(*cs->clearTargetOutput) (s, rRotate, vRotate);
 	cs->cleared[output] = TRUE;
     }
 
@@ -1480,6 +1476,8 @@ cubePaintTransformedOutput (CompScreen		    *s,
     {
 	/* Outside cube - start with FTB faces */
 	paintOrder = FTB;
+	if (wasCulled && cs->desktopOpacity != OPAQUE)
+	    glDisable (GL_CULL_FACE);
     }
     else
     {
@@ -1492,7 +1490,12 @@ cubePaintTransformedOutput (CompScreen		    *s,
 			       outputPtr, mask, xMove,
 			       size, hsize, paintOrder);
 
-    if (cs->grabIndex == 0 && hsize > 2 &&
+    if (wasCulled && cs->desktopOpacity != OPAQUE)
+	glDisable (GL_CULL_FACE);
+
+    capsPainted = cs->capsPainted[output];
+
+    if (cs->grabIndex == 0 && hsize > 2 && !capsPainted &&
 	(cs->invert != 1 || cs->desktopOpacity != OPAQUE ||
 	 sa.vRotate != 0.0f || sa.yTranslate != 0.0f))
     {
@@ -1505,6 +1508,8 @@ cubePaintTransformedOutput (CompScreen		    *s,
 	topDir    = cs->checkOrientation(s, &sa, transform, outputPtr, top);
 	bottomDir = cs->checkOrientation(s, &sa, transform, outputPtr, bottom);
 
+	cs->capsPainted[output] = TRUE;
+
 	if (topDir && bottomDir)
 	{
 	    glNormal3f (0.0f, -1.0f, 0.0f);
@@ -1545,6 +1550,8 @@ cubePaintTransformedOutput (CompScreen		    *s,
     {
 	/* Outside cube - continue with BTF faces */
 	paintOrder = BTF;
+	if (wasCulled && cs->desktopOpacity != OPAQUE)
+	    glEnable (GL_CULL_FACE);
     }
     else
     {
@@ -1937,7 +1944,7 @@ cubeOutputChangeNotify (CompScreen *s)
 
 static Bool
 cubeSetGlobalScreenOption (CompScreen      *s,
-			   const char	   *name,
+			   char		   *name,
 			   CompOptionValue *value)
 {
     Bool status;
@@ -1971,7 +1978,7 @@ cubeGetDisplayOptions (CompPlugin  *plugin,
 static Bool
 cubeSetDisplayOption (CompPlugin      *plugin,
 		      CompDisplay     *display,
-		      const char      *name,
+		      char	      *name,
 		      CompOptionValue *value)
 {
     CompOption *o;
@@ -1997,10 +2004,10 @@ cubeSetDisplayOption (CompPlugin      *plugin,
 static const CompMetadataOptionInfo cubeDisplayOptionInfo[] = {
     { "abi", "int", 0, 0, 0 },
     { "index", "int", 0, 0, 0 },
-    { "unfold_key", "key", 0, cubeUnfold, cubeFold },
-    { "next_slide_key", "key", "<passive_grab>false</passive_grab>",
+    { "unfold", "action", 0, cubeUnfold, cubeFold },
+    { "next_slide", "action", "<passive_grab>false</passive_grab>",
       cubeNextImage, 0 },
-    { "prev_slide_key", "key", "<passive_grab>false</passive_grab>",
+    { "prev_slide", "action", "<passive_grab>false</passive_grab>",
       cubePrevImage, 0 }
 };
 
@@ -2192,6 +2199,9 @@ cubeFiniScreen (CompPlugin *p,
 {
     CUBE_SCREEN (s);
 
+    if (cs->vertices)
+	free (cs->vertices);
+
     if (cs->skyListId)
 	glDeleteLists (cs->skyListId, 1);
 
