@@ -63,20 +63,19 @@ struct _ResizeKeys {
 #define MIN_KEY_WIDTH_INC  24
 #define MIN_KEY_HEIGHT_INC 24
 
-#define RESIZE_DISPLAY_OPTION_INITIATE_NORMAL_KEY    0
-#define RESIZE_DISPLAY_OPTION_INITIATE_OUTLINE_KEY   1
-#define RESIZE_DISPLAY_OPTION_INITIATE_RECTANGLE_KEY 2
-#define RESIZE_DISPLAY_OPTION_INITIATE_STRETCH_KEY   3
-#define RESIZE_DISPLAY_OPTION_INITIATE_BUTTON	     4
-#define RESIZE_DISPLAY_OPTION_INITIATE_KEY	     5
-#define RESIZE_DISPLAY_OPTION_MODE	             6
-#define RESIZE_DISPLAY_OPTION_BORDER_COLOR           7
-#define RESIZE_DISPLAY_OPTION_FILL_COLOR             8
-#define RESIZE_DISPLAY_OPTION_NORMAL_MATCH	     9
-#define RESIZE_DISPLAY_OPTION_OUTLINE_MATCH	     10
-#define RESIZE_DISPLAY_OPTION_RECTANGLE_MATCH	     11
-#define RESIZE_DISPLAY_OPTION_STRETCH_MATCH	     12
-#define RESIZE_DISPLAY_OPTION_NUM		     13
+#define RESIZE_DISPLAY_OPTION_INITIATE_NORMAL    0
+#define RESIZE_DISPLAY_OPTION_INITIATE_OUTLINE   1
+#define RESIZE_DISPLAY_OPTION_INITIATE_RECTANGLE 2
+#define RESIZE_DISPLAY_OPTION_INITIATE_STRETCH   3
+#define RESIZE_DISPLAY_OPTION_INITIATE	         4
+#define RESIZE_DISPLAY_OPTION_MODE	         5
+#define RESIZE_DISPLAY_OPTION_BORDER_COLOR       6
+#define RESIZE_DISPLAY_OPTION_FILL_COLOR         7
+#define RESIZE_DISPLAY_OPTION_NORMAL_MATCH	 8
+#define RESIZE_DISPLAY_OPTION_OUTLINE_MATCH	 9
+#define RESIZE_DISPLAY_OPTION_RECTANGLE_MATCH	 10
+#define RESIZE_DISPLAY_OPTION_STRETCH_MATCH	 11
+#define RESIZE_DISPLAY_OPTION_NUM		 12
 
 static int displayPrivateIndex;
 
@@ -324,7 +323,6 @@ resizeInitiate (CompDisplay     *d,
     w = findWindowAtDisplay (d, xid);
     if (w && (w->actions & CompWindowActionResizeMask))
     {
-	unsigned int mods;
 	unsigned int mask;
 	int          x, y;
 	int	     button;
@@ -332,8 +330,6 @@ resizeInitiate (CompDisplay     *d,
 
 	RESIZE_SCREEN (w->screen);
 
-	mods = getIntOptionNamed (option, nOption, "modifiers", 0);
-
 	x = getIntOptionNamed (option, nOption, "x",
 			       w->serverX + (w->serverWidth / 2));
 	y = getIntOptionNamed (option, nOption, "y",
@@ -394,29 +390,39 @@ resizeInitiate (CompDisplay     *d,
 	rd->pointerDx = x - pointerX;
 	rd->pointerDy = y - pointerY;
 
-	rd->mode = rd->opt[RESIZE_DISPLAY_OPTION_MODE].value.i;
-	for (i = 0; i <= RESIZE_MODE_LAST; i++)
+	if ((w->state & MAXIMIZE_STATE) == MAXIMIZE_STATE)
 	{
-	    if (action == &rd->opt[i].value.action)
-	    {
-		rd->mode = i;
-		break;
-	    }
+	    /* if the window is fully maximized, showing the outline or
+	       rectangle would be visually distracting as the window can't
+	       be resized anyway; so we better don't use them in this case */
+	    rd->mode = RESIZE_MODE_NORMAL;
 	}
-
-	if (i > RESIZE_MODE_LAST)
+	else
 	{
-	    int index;
-
+	    rd->mode = rd->opt[RESIZE_DISPLAY_OPTION_MODE].value.i;
 	    for (i = 0; i <= RESIZE_MODE_LAST; i++)
 	    {
-		index = RESIZE_DISPLAY_OPTION_NORMAL_MATCH + i;
-		if (matchEval (&rd->opt[index].value.match, w))
+		if (action == &rd->opt[i].value.action)
 		{
 		    rd->mode = i;
 		    break;
 		}
 	    }
+
+	    if (i > RESIZE_MODE_LAST)
+	    {
+		int index;
+
+		for (i = 0; i <= RESIZE_MODE_LAST; i++)
+		{
+		    index = RESIZE_DISPLAY_OPTION_NORMAL_MATCH + i;
+		    if (matchEval (&rd->opt[index].value.match, w))
+		    {
+			rd->mode = i;
+			break;
+		    }
+		}
+	    }
 	}
 
 	if (!rs->grabIndex)
@@ -702,13 +708,12 @@ resizeHandleMotionEvent (CompScreen *s,
 		CompAction *action;
 		int        pointerAdjustX = 0;
 		int        pointerAdjustY = 0;
-		int	   option = RESIZE_DISPLAY_OPTION_INITIATE_KEY;
 
 		RESIZE_SCREEN (s);
 
-		action = &rd->opt[option].value.action;
+		action = &rd->opt[RESIZE_DISPLAY_OPTION_INITIATE].value.action;
 		action->state |= CompActionStateTermButton;
-
+	
 		if (rd->mask & ResizeRightMask)
 			pointerAdjustX = w->serverX + w->serverWidth +
 					 w->input.right - xRoot;
@@ -730,7 +735,7 @@ resizeHandleMotionEvent (CompScreen *s,
 	else
 	{
 	    /* only accumulate pointer movement if a mask is
-	       already set as we don't have a use for the
+	       already set as we don't have a use for the 
 	       difference information otherwise */
 	    rd->pointerDx += xRoot - lastPointerX;
 	    rd->pointerDy += yRoot - lastPointerY;
@@ -810,7 +815,7 @@ resizeHandleEvent (CompDisplay *d,
 	break;
     case ButtonRelease: {
 	CompAction *action =
-	    &rd->opt[RESIZE_DISPLAY_OPTION_INITIATE_BUTTON].value.action;
+	    &rd->opt[RESIZE_DISPLAY_OPTION_INITIATE].value.action;
 
 	if (action->state & CompActionStateTermButton)
 	{
@@ -849,7 +854,8 @@ resizeHandleEvent (CompDisplay *d,
 		if (w)
 		{
 		    CompOption o[6];
-		    int	       option;
+		    CompAction *action =
+			&rd->opt[RESIZE_DISPLAY_OPTION_INITIATE].value.action;
 
 		    o[0].type    = CompOptionTypeInt;
 		    o[0].name    = "window";
@@ -857,13 +863,11 @@ resizeHandleEvent (CompDisplay *d,
 
 		    if (event->xclient.data.l[2] == WmMoveResizeSizeKeyboard)
 		    {
-			option = RESIZE_DISPLAY_OPTION_INITIATE_KEY;
-
 			o[1].type    = CompOptionTypeInt;
 			o[1].name    = "button";
 			o[1].value.i = 0;
 
-			resizeInitiate (d, &rd->opt[option].value.action,
+			resizeInitiate (d, action,
 					CompActionStateInitKey,
 					o, 2);
 		    }
@@ -883,8 +887,6 @@ resizeHandleEvent (CompDisplay *d,
 			Window	     root, child;
 			int	     xRoot, yRoot, i;
 
-			option = RESIZE_DISPLAY_OPTION_INITIATE_BUTTON;
-
 			XQueryPointer (d->display, w->screen->root,
 				       &root, &child, &xRoot, &yRoot,
 				       &i, &i, &mods);
@@ -914,7 +916,7 @@ resizeHandleEvent (CompDisplay *d,
 				event->xclient.data.l[3] : -1;
 
 			    resizeInitiate (d,
-					    &rd->opt[option].value.action,
+					    action,
 					    CompActionStateInitButton,
 					    o, 6);
 
@@ -928,23 +930,19 @@ resizeHandleEvent (CompDisplay *d,
     case DestroyNotify:
 	if (rd->w && rd->w->id == event->xdestroywindow.window)
 	{
-	    int option;
+	    CompAction *action =
+		&rd->opt[RESIZE_DISPLAY_OPTION_INITIATE].value.action;
 
-	    option = RESIZE_DISPLAY_OPTION_INITIATE_BUTTON;
-	    resizeTerminate (d, &rd->opt[option].value.action, 0, NULL, 0);
-	    option = RESIZE_DISPLAY_OPTION_INITIATE_KEY;
-	    resizeTerminate (d, &rd->opt[option].value.action, 0, NULL, 0);
+	    resizeTerminate (d, action, 0, NULL, 0);
 	}
 	break;
     case UnmapNotify:
 	if (rd->w && rd->w->id == event->xunmap.window)
 	{
-	    int option;
+	    CompAction *action =
+		&rd->opt[RESIZE_DISPLAY_OPTION_INITIATE].value.action;
 
-	    option = RESIZE_DISPLAY_OPTION_INITIATE_BUTTON;
-	    resizeTerminate (d, &rd->opt[option].value.action, 0, NULL, 0);
-	    option = RESIZE_DISPLAY_OPTION_INITIATE_KEY;
-	    resizeTerminate (d, &rd->opt[option].value.action, 0, NULL, 0);
+	    resizeTerminate (d, action, 0, NULL, 0);
 	}
     default:
 	break;
@@ -1042,7 +1040,7 @@ resizePaintOutput (CompScreen              *s,
     RESIZE_SCREEN (s);
     RESIZE_DISPLAY (s->display);
 
-    if (rd->w)
+    if (rd->w && (s == rd->w->screen))
     {
 	if (rd->mode == RESIZE_MODE_STRETCH)
 	    mask |= PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS_MASK;
@@ -1052,7 +1050,7 @@ resizePaintOutput (CompScreen              *s,
     status = (*s->paintOutput) (s, sAttrib, transform, region, output, mask);
     WRAP (rs, s, paintOutput, resizePaintOutput);
 
-    if (status && rd->w)
+    if (status && rd->w && (s == rd->w->screen))
     {
 	unsigned short *border, *fill;
 
@@ -1179,7 +1177,7 @@ resizeGetDisplayOptions (CompPlugin  *plugin,
 static Bool
 resizeSetDisplayOption (CompPlugin      *plugin,
 			CompDisplay     *display,
-			const char	*name,
+			char	        *name,
 			CompOptionValue *value)
 {
     CompOption *o;
@@ -1194,12 +1192,11 @@ resizeSetDisplayOption (CompPlugin      *plugin,
 }
 
 static const CompMetadataOptionInfo resizeDisplayOptionInfo[] = {
-    { "initiate_normal_key", "key", 0, resizeInitiate, resizeTerminate },
-    { "initiate_outline_key", "key", 0, resizeInitiate, resizeTerminate },
-    { "initiate_rectangle_key", "key", 0, resizeInitiate, resizeTerminate },
-    { "initiate_stretch_key", "key", 0, resizeInitiate, resizeTerminate },
-    { "initiate_button", "button", 0, resizeInitiate, resizeTerminate },
-    { "initiate_key", "key", 0, resizeInitiate, resizeTerminate },
+    { "initiate_normal", "action", 0, resizeInitiate, resizeTerminate },
+    { "initiate_outline", "action", 0, resizeInitiate, resizeTerminate },
+    { "initiate_rectangle", "action", 0, resizeInitiate, resizeTerminate },
+    { "initiate_stretch", "action", 0, resizeInitiate, resizeTerminate },
+    { "initiate", "action", 0, resizeInitiate, resizeTerminate },
     { "mode", "int", RESTOSTRING (0, RESIZE_MODE_LAST), 0, 0 },
     { "border_color", "color", 0, 0, 0 },
     { "fill_color", "color", 0, 0, 0 },
