@@ -207,9 +207,9 @@ videoGetDisplayOptions (CompPlugin  *plugin,
 }
 
 static Bool
-videoSetDisplayOption (CompPlugin      *plugin,
+videoSetDisplayOption (CompPlugin  *plugin,
 		       CompDisplay     *display,
-		       const char      *name,
+		       char	       *name,
 		       CompOptionValue *value)
 {
     CompOption *o;
