@@ -1595,8 +1595,11 @@ handleEvent (CompDisplay *d,
 	    {
 		/* use focus stealing prevention if request came from an
 		   application (which means data.l[0] is 1 */
-		if (event->xclient.data.l[0] != 1 || allowWindowFocus (w))
+		if (event->xclient.data.l[0] != 1 ||
+		    allowWindowFocus (w, 0, event->xclient.data.l[1]))
+		{
 		    activateWindow (w);
+		}
 	    }
 	}
 	else if (event->xclient.message_type == d->winOpacityAtom)
@@ -1920,16 +1923,14 @@ handleEvent (CompDisplay *d,
 		    w->placed   = TRUE;
 		}
 
-		allowFocus = allowWindowFocus (w);
+		allowFocus = allowWindowFocus (w, NO_FOCUS_MASK, 0);
 
-		if (allowFocus)
-		    updateWindowAttributes (w, CompStackingUpdateModeNormal);
-		else
-		    updateWindowAttributes (w, CompStackingUpdateModeNone);
+		updateWindowAttributes (w, CompStackingUpdateModeInitialMap);
 
 		if (!allowFocus &&
 		    (w->type & ~(CompWindowTypeSplashMask |
-				 CompWindowTypeDockMask)))
+				 CompWindowTypeDockMask   |
+				 CompWindowTypeDesktopMask)))
 		{
 		    CompWindow *p;
 
@@ -1982,12 +1983,18 @@ handleEvent (CompDisplay *d,
 
 	    if (event->xconfigurerequest.value_mask & CWStackMode)
 	    {
-		Window     above = event->xconfigure.above;
-		CompWindow *sibling = findWindowAtDisplay (d, above);
+		Window     above    = None;
+		CompWindow *sibling = NULL;
+
+		if (event->xconfigurerequest.value_mask & CWSibling)
+		{
+		    above   = event->xconfigurerequest.above;
+		    sibling = findTopLevelWindowAtDisplay (d, above);
+		}
 
 		switch (event->xconfigurerequest.detail) {
 		case Above:
-		    if (allowWindowFocus (w))
+		    if (allowWindowFocus (w, NO_FOCUS_MASK, 0))
 		    {
 			if (above)
 			{
