@@ -1785,6 +1785,12 @@ addScreen (CompDisplay *display,
     currentRoot = s->root;
 
     glExtensions = (const char *) glGetString (GL_EXTENSIONS);
+    if (!glExtensions)
+    {
+	compLogMessage (display, "core", CompLogLevelFatal,
+			"No valid GL extensions string found.");
+	return FALSE;
+    }
 
     s->textureNonPowerOfTwo = 0;
     if (strstr (glExtensions, "GL_ARB_texture_non_power_of_two"))
@@ -3118,8 +3124,42 @@ runCommand (CompScreen *s,
 
     if (fork () == 0)
     {
+	/* build a display string that uses the right screen number */
+	/* 5 extra chars should be enough for pretty much every situation */
+	int  stringLen = strlen (s->display->displayString) + 5;
+	char screenString[stringLen];
+	char *pos, *delimiter, *colon;
+	
 	setsid ();
-	putenv (s->display->displayString);
+
+	if (screenString)
+	{
+	    strcpy (screenString, s->display->displayString);
+	    delimiter = strrchr (screenString, ':');
+	    if (delimiter)
+	    {
+		colon = "";
+		delimiter = strchr (delimiter, '.');
+		if (delimiter)
+		    *delimiter = '\0';
+	    }
+	    else
+	    {
+		/* insert :0 to keep the syntax correct */
+		colon = ":0";
+	    }
+	    pos = screenString + strlen (screenString);
+
+	    snprintf (pos, stringLen - (pos - screenString),
+		      "%s.%d", colon, s->screenNum);
+
+	    putenv (screenString);
+	}
+	else
+	{
+	    putenv (s->display->displayString);
+	}
+
 	execl ("/bin/sh", "/bin/sh", "-c", command, NULL);
 	exit (0);
     }
